open Format

exception E of Pos.range * string

let print range msg =
  Pos.print_range stderr range;
  prerr_string "Error. ";
  prerr_endline msg

let mk range =
  kfprintf begin fun _ ->
    raise (E (range, flush_str_formatter ()))
  end str_formatter

let unguarded_variables range vars =
  mk range "Unguarded variables: %a" (Var.Set.format ", " Var.format) vars

let some_variables_only_appear_in_one_branch range vars =
  mk range "Some variables only appear in one branch: %a"
    (Var.Set.format ", " Var.format) vars

let invalid_field range =
  mk range "Invalid field."

let invalid_field_name range =
  mk range "Invalid field name."

let duplicated_field range s =
  mk range "Duplicated field: \"%s\"." s

let reserved_field_name range s =
  mk range "Reserved field name: \"%s\"." s

let unknown_char range s =
  mk range "Unknown character: \"%s\"." s

let syntax range =
  mk range "Syntax error."

let unbound_variable range v =
  mk range "Unbound variable: \"%s\"." v

let should_be_a_variable range =
  mk range "Should be a variable."

let invalid_agent range =
  mk range "Invalid agent."

let invalid_constraint range =
  mk range "Invalid constraint."

let invalid_value range =
  mk range "Invalid value."

let constraint_always_inconsistent range =
  mk range "Constraint always inconsistent."

let module_name_mismatch range name =
  mk range "This module has name '%s'." name

let char_expected range =
  mk range "Character expected."

let unknown_escaped_char range =
  mk range "Unknown escaped character."

let unterminated_string range =
  mk range "Unterminated string."

let unterminated_comment range =
  mk range "Unterminated comment."

let should_be_equality range =
  mk range "Should be an equality."

let persistent_ask_guard_cannot_be_empty range =
  mk range "Persistent ask guard cannot be empty."

let persistent_ask_guard_should_begin_by_a_token range =
  mk range "Persistent ask should begin by a token."

let unknown_builtin range f arity =
  mk range "Unknown builtin %s/%d" f arity

let unmatched_delimiter range delim =
  mk range "Unmatched delimiter: '%s'." delim

let mismatched_delimiter range delim_start delim_end =
  mk range "Mismatched delimiters: '%s' and '%s'." delim_start delim_end

let should_be_empty range =
  mk range "Should be empty"

let mismatch_xml_tag_names range =
  mk range "Mismatch XML tag names."

let missing_xml_tag_name range =
  mk range "Missing XML tag name."

let usage () =
  eprintf "Try `%s --help' for more information.@." Sys.argv.(0)

let missing_option_argument s =
  eprintf "Missing argument for the option '%s'.@." s;
  usage ();
  exit 1

let unknown_option s =
  eprintf "Unknown option: %s.@." s;
  usage ();
  exit 1

let unknown_optimization s =
  eprintf "Unknown optimization: %s.@." s;
  usage ();
  exit 1

let option_does_not_expect_argument s =
  eprintf "The option '%s' does not expect any argument.@." s;
  usage ();
  exit 1

let exhausted () =
  eprintf "@.Exhausted.@.";
  exit 1

let invalid_package_name filename =
  eprintf "The file '%s' has an invalid package name.@." filename;
  exit 1

let module_not_found filename =
  eprintf "Module '%s' not found.@." filename;
  exit 1
